import 'dart:async';

import 'package:aitrainer_app/bloc/account/account_bloc.dart';
import 'package:aitrainer_app/model/cache.dart';
import 'package:aitrainer_app/model/customer.dart';
import 'package:aitrainer_app/repository/customer_repository.dart';
import 'package:aitrainer_app/repository/mautic_repository.dart';
import 'package:aitrainer_app/repository/split_test_respository.dart';
import 'package:aitrainer_app/repository/training_plan_repository.dart';
import 'package:aitrainer_app/repository/user_repository.dart';
import 'package:aitrainer_app/util/app_language.dart';
import 'package:aitrainer_app/util/common.dart';
import 'package:aitrainer_app/util/enums.dart';
import 'package:aitrainer_app/util/track.dart';
import 'package:aitrainer_app/util/trans.dart';
import 'package:bloc/bloc.dart';
import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';

part 'login_event.dart';
part 'login_state.dart';

class LoginBloc extends Bloc<LoginEvent, LoginState> with Trans {
  final AccountBloc accountBloc;
  final UserRepository userRepository;
  late CustomerRepository? customerRepository;
  final SplitTestRepository splitTestRepository = SplitTestRepository();
  final BuildContext context;
  final bool isRegistration;
  bool dataPolicyAllowed = false;
  bool emailSubscription = false;
  bool obscure = true;

  Color testColor = Colors.green[800]!;
  bool emailCheckbox = true;

  LoginBloc(
      {required this.accountBloc,
      required this.userRepository,
      required this.context,
      required this.isRegistration,
      this.customerRepository})
      : super(LoginInitial()) {
    String colorString = splitTestRepository.getSplitTestValue("registration_skip");
    if (colorString == "red") {
      testColor = Colors.red[800]!;
    }
    String emailCheckboxString = splitTestRepository.getSplitTestValue("email_checkbox");
    if (emailCheckboxString == "0") {
      emailCheckbox = false;
    }
    if (customerRepository == null) {
      customerRepository = CustomerRepository();
    }
  }

  @override
  Stream<LoginState> mapEventToState(
    LoginEvent event,
  ) async* {
    try {
      if (event is LoginEmailChange) {
        yield LoginLoading();
        final String email = event.email;
        userRepository.setEmail(email);
        yield LoginReady();
      } else if (event is LoginPasswordChange) {
        yield LoginLoading();
        final String password = event.password;
        userRepository.setPassword(password);
        yield LoginReady();
      } else if (event is LoginSubmit) {
        yield LoginLoading();
        await userRepository.getUser();
        accountBloc.add(AccountLogInFinished(customer: Cache().userLoggedIn!));
        Track().track(TrackingEvent.login, eventValue: "email");
        Cache().setLoginType(LoginType.email);
        yield LoginSuccess();
      } else if (event is LoginFB) {
        yield LoginLoading();
        Cache().setLoginType(LoginType.fb);
        await userRepository.getUserByFB();
        accountBloc.add(AccountLogInFinished(customer: Cache().userLoggedIn!));
        Track().track(TrackingEvent.login, eventValue: "FB");
        yield LoginSuccess();
      } else if (event is LoginGoogle) {
        yield LoginLoading();
        Cache().setLoginType(LoginType.google);
        await userRepository.getUserByGoogle();
        accountBloc.add(AccountLogInFinished(customer: Cache().userLoggedIn!));
        Track().track(TrackingEvent.login, eventValue: "Google");
        yield LoginSuccess();
      } else if (event is LoginApple) {
        yield LoginLoading();
        Cache().setLoginType(LoginType.apple);
        await userRepository.getUserByApple();
        accountBloc.add(AccountLogInFinished(customer: Cache().userLoggedIn!));
        Track().track(TrackingEvent.login, eventValue: "Apple");
        yield LoginSuccess();
      } else if (event is RegistrationSubmit) {
        yield LoginLoading();

        final String? validationError = validate();
        if (validationError != null) {
          yield LoginError(message: validationError);
        } else {
          await userRepository.addUser();
          accountBloc.add(AccountLogInFinished(customer: Cache().userLoggedIn!));
          customerRepository!.customer!.emailSubscription = emailSubscription == true ? 1 : 0;
          await afterRegistration("email");
          Cache().setLoginType(LoginType.email);
          yield LoginSuccess();
        }
      } else if (event is RegistrationFB) {
        yield LoginLoading();

        Cache().setLoginType(LoginType.fb);
        await userRepository.addUserFB();
        accountBloc.add(AccountLogInFinished(customer: Cache().userLoggedIn!));
        customerRepository!.customer!.emailSubscription = emailSubscription == true ? 1 : 0;
        await afterRegistration("FB");
        yield LoginSuccess();
      } else if (event is RegistrationGoogle) {
        yield LoginLoading();

        Cache().setLoginType(LoginType.google);
        await userRepository.addUserGoogle();
        accountBloc.add(AccountLogInFinished(customer: Cache().userLoggedIn!));
        customerRepository!.customer!.emailSubscription = emailSubscription == true ? 1 : 0;
        await afterRegistration("Google");
        yield LoginSuccess();
      } else if (event is RegistrationApple) {
        yield LoginLoading();

        Cache().setLoginType(LoginType.apple);
        await userRepository.addUserApple();
        accountBloc.add(AccountLogInFinished(customer: Cache().userLoggedIn!));
        customerRepository!.customer!.emailSubscription = emailSubscription == true ? 1 : 0;
        await afterRegistration("Apple");

        yield LoginSuccess();
      } else if (event is DataProtectionClicked) {
        yield LoginLoading();
        this.dataPolicyAllowed = !dataPolicyAllowed;
        yield LoginReady();
      } else if (event is EmailSubscriptionClicked) {
        yield LoginLoading();
        this.emailSubscription = !emailSubscription;
        yield LoginReady();
      } else if (event is LoginPasswordChangeObscure) {
        yield LoginLoading();
        this.obscure = !this.obscure;
        yield LoginReady();
      } else if (event is LoginSkip) {
        yield LoginLoading();
        Track().track(TrackingEvent.login_skip);
        Cache().startPage = "home";
        yield LoginSkipped();
      }
    } on Exception catch (e) {
      yield LoginError(message: e.toString());
    }
  }

  Future<void> afterRegistration(String event) async {
    Customer tempCustomer = customerRepository!.customer!;
    customerRepository!.customer = Cache().userLoggedIn!;
    customerRepository!.customer!.fitnessLevel = tempCustomer.fitnessLevel;
    customerRepository!.customer!.goal = tempCustomer.goal;
    customerRepository!.customer!.sex = tempCustomer.sex;
    customerRepository!.customer!.dataPolicyAllowed = 1;
    customerRepository!.customer!.emailSubscription = 1;
    customerRepository!.customer!.syncedDate = DateTime.now();
    customerRepository!.customer!.lang = AppLanguage().appLocal.languageCode;
    customerRepository!.setCustomerProperty("Weight", tempCustomer.getProperty("Weight"));
    customerRepository!.setCustomerProperty("Height", tempCustomer.getProperty("Height"));
    await customerRepository!.saveCustomer();
    MauticRepository mauticRepository = MauticRepository(customerRepository: customerRepository!);
    await mauticRepository.sendMauticSubscription();
    TrainingPlanRepository trainingPlanRepository = TrainingPlanRepository();
    trainingPlanRepository.generateTrainingPlan();
    Track().track(TrackingEvent.registration, eventValue: event);
  }

  String? emailValidation(String? email) {
    String? message = Common.emailValidation(email);
    return message;
  }

  String? passwordValidation(String? value) {
    String? message = Common.passwordValidation(value);
    if (message != null) {
      message = t(message);
    }
    return message;
  }

  String? validate() {
    String? error;

    error = emailValidation(userRepository.user.email);
    if (error != null) {
      return error;
    }

    error = passwordValidation(userRepository.user.password);
    if (error != null) {
      return error;
    }

    return error;
  }
}
